/*
    Copyright (c) 2010 digitalSTROM.org, Zurich, Switzerland

    Author: Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc> 

    This file is part of upnp-beacon.

    upnp-beacon is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as 
    published by the Free Software Foundation.

    upnp-beacon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with upnp-beacon. If not, see <http://www.gnu.org/licenses/>.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "devdesc.h"

#define STR1 "<?xml version=\"1.0\"?><root xmlns=\"urn:schemas-upnp-org:device-1-0\"><specVersion><major>1</major><minor>0</minor></specVersion><device><deviceType>upnp:rootdevice</deviceType><friendlyName>"

#define STR2 "</friendlyName><manufacturer>"
#define STR3 "</manufacturer><manufacturerURL>"
#define STR4 "</manufacturerURL><modelDescription>"
#define STR5 "</modelDescription><modelName>"
#define STR6 "</modelName><UDN>"
#define STR7 "</UDN><presentationURL>"
#define STR8 "</presentationURL><iconList><icon><mimetype>image/png</mimetype><width>32</width><height>32</height><depth>24</depth><url>icon32.png</url></icon><icon><mimetype>image/png</mimetype><width>48</width><height>48</height><depth>24</depth><url>icon48.png</url></icon><icon><mimetype>image/png</mimetype><width>120</width><height>120</height><depth>24</depth><url>icon120.png</url></icon></iconList></device></root>"

#define PROTO_SEP "://"
#define PORT_SEP  ":"
#define URL_SEP   "/"
#define UUID      "uuid:"

char *device_desc_from_config(const desc_config_t *config)
{
    char *buffer = NULL;

    if ((config->friendly_name == NULL) || (config->manufacturer == NULL) ||
        (config->manufacturer_url == NULL) || (config->modeldesc == NULL) ||
        (config->modelname == NULL) || (config->udn == NULL) ||
        (config->url == NULL) || (config->ip == NULL) || 
        (config->protocol == NULL) || (config->port == NULL))
    {
        fprintf(stderr, "Missing parameters in device description "
                        "configuration\n");
        return NULL;
    }

    size_t buflen = strlen(STR1 STR2 STR3 STR4 STR5 STR6 STR7 STR8 PROTO_SEP 
                           PORT_SEP URL_SEP UUID) + 
                    strlen(config->friendly_name) + 
                    strlen(config->manufacturer) +
                    strlen(config->manufacturer_url) +
                    strlen(config->modeldesc) +
                    strlen(config->modelname) + 
                    strlen(config->udn) + 
                    strlen(config->protocol) +
                    strlen(config->ip) +
                    strlen(config->port) + 
                    strlen(config->url) + 1;

    buffer = (char *)malloc(buflen*sizeof(char));
    if (buffer == NULL)
    {
        fprintf(stderr, "Could not allocate memory for device description "
                        "document: %s\n", strerror(errno));
        return NULL;
    }
  
    snprintf(buffer, buflen,
                    STR1 "%s" STR2 "%s" STR3 "%s" STR4 "%s"STR5 "%s" 
                    STR6 UUID "%s" STR7 "%s" PROTO_SEP "%s" PORT_SEP "%s" 
                    URL_SEP "%s" STR8,
                    config->friendly_name, config->manufacturer, 
                    config->manufacturer_url, config->modeldesc, config->modelname,
                    config->udn, config->protocol, config->ip, config->port,
                    config->url);
    return buffer;
}

